<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2013 IMIA net based solutions (d.frerich@imia.de)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

namespace IMIA\ImiaPageteaser\Controller;

use IMIA\ImiaBaseExt\Utility\Helper;
use IMIA\ImiaBaseExt\Utility\Session\Storage\SessionStorage;
use IMIA\ImiaPageteaser\Domain\Repository\CategoryRepository;
use IMIA\ImiaPageteaser\Domain\Repository\PageAltRepository;
use IMIA\ImiaPageteaser\Domain\Repository\PageRepository;
use \TYPO3\CMS\Core\Utility\GeneralUtility;
use \IMIA\ImiaBaseExt\Utility\Cache;

/**
 * @package     imia_pageteaser
 * @subpackage  Controller
 * @author      David Frerich <d.frerich@imia.de>
 * @author      Markus Ehrlich <m.ehrlich@imia.de>
 * @author      Sascha Lorenz <s.lorenz@imia.de>
 */
class TeaserController extends \IMIA\ImiaBaseExt\Controller\ActionController
{
    /**
     * @var array
     */
    static protected $lastModified = array();

    /**
     * @var Cache
     */
    static protected $cache;

    /**
     * @var integer
     */
    protected $limit;

    /**
     * @var string
     */
    protected $sessionDataKey;

    /**
     * @var array
     */
    protected $doktypes;

    /**
     * @var array
     */
    protected $categories;

    /**
     * @var array
     */
    protected $filter;

    /**
     * @var integer
     */
    protected $pageNum;

    /**
     * @var integer
     */
    protected $pageUid;

    /**
     * @var array
     */
    protected $contentData;

    /**
     * @var boolean
     */
    protected $allowParamOverwrite = false;

    /**
     * @var SessionStorage
     */
    protected $sessionStorage;

    /**
     * @var PageRepository
     */
    protected $pageRepository;

    /**
     * @var PageAltRepository
     */
    protected $pageAltRepository;

    /**
     * @var CategoryRepository
     */
    protected $categoryRepository;

    /**
     * @param SessionStorage $sessionStorage
     */
    public function injectSessionStorage(SessionStorage $sessionStorage)
    {
        $this->sessionStorage = $sessionStorage;
    }

    /**
     * @param PageRepository $pageRepository
     */
    public function injectPageRepository(PageRepository $pageRepository)
    {
        $this->pageRepository = $pageRepository;
    }

    /**
     * @param PageAltRepository $pageAltRepository
     */
    public function injectPageAltRepository(PageAltRepository $pageAltRepository)
    {
        $this->pageAltRepository = $pageAltRepository;
    }

    /**
     * @param CategoryRepository $categoryRepository
     */
    public function injectCategoryRepository(CategoryRepository $categoryRepository)
    {
        $this->categoryRepository = $categoryRepository;
    }

    public function indexAction()
    {
        $this->view->assign('indexPage', $this->getPage());
        $this->view->assign('contentObject', $this->getContentData());

        $filter = array();
        $filterOptions = array();
        if (!$this->getSetting('disableFilter')) {
            $filter = $this->getFilter();
            $filterOptions = $this->getFilterOptions();
        }

        $this->view->assign('filter', $filter);
        $this->view->assign('filterOptions', $filterOptions);
        $this->view->assign('categories', $this->getCategories());

        if ($this->getLimit() > 0) {
            $result = $this->getResult();
            $this->view->assign('pages', $this->getPages($result));
            $this->view->assign('pageNum', $this->getPageNum());
            $this->view->assign('pageCount', $this->getPagesCount($result));
        }
    }

    public function navigationAction()
    {
        $this->allowParamOverwrite = true;
        $this->updateSettings();

        $result = $this->getResult();
        $pageUid = $GLOBALS['TSFE']->id;
        $pageFound = false;
        $prev = null;
        $next = null;

        $backPid = (int)GeneralUtility::_GP('backPid');
        $overview = $backPid ?: $GLOBALS['TSFE']->page['pid'];

        foreach($result as $uid) {
            if ($pageFound) {
                $next = $uid;
                break;
            } elseif ($uid == $pageUid) {
                $pageFound = true;
            } else {
                $prev = $uid;
            }
        }

        $additionalParams = array();
        if ($backPid) {
            $additionalParams['backPid'] = $backPid;
        }
        if ($this->getContentUid() && $this->getContentUid() != $this->configurationManager->getContentObject()->data['uid']) {
            $additionalParams['contentUid'] = $this->getContentUid();
        }

        $this->view->assign('prev', $this->pageRepository->findOneByUid((int)$prev));
        $this->view->assign('next', $this->pageRepository->findOneByUid((int)$next));
        $this->view->assign('overview', $this->pageRepository->findOneByUid((int)$overview));
        $this->view->assign('additionalParams', $additionalParams);
    }

    public function categoriesAction()
    {
        $this->allowParamOverwrite = true;
        $this->updateSettings();

        $filter = array();
        $filterOptions = array();
        if (!$this->getSetting('disableFilter')) {
            $filter = $this->getFilter();
            $filterOptions = $this->getFilterOptions();
        }

        $this->view->assign('filter', $filter);
        $this->view->assign('filterOptions', $filterOptions);
        $this->view->assign('categories', $this->getCategories());
        $this->view->assign('listContentUid', $this->getSetting('listContentUid'));
    }

    /**
     * @return array
     */
    public function getResult()
    {
        $result = null;
        $sessionData = $this->getSessionData();
        if (array_key_exists('results', $sessionData)
          && array_key_exists($this->getLang(), $sessionData['results'])) {
            $result = $sessionData['results'][$this->getLang()];
        }

        if ($result && !array_key_exists('modified', $sessionData)
            || !is_array($sessionData['modified']) || !array_key_exists('content', $sessionData['modified'])
            || $sessionData['modified']['content'] < (int)$this->getContentObject()->data['tstamp']) {
            $sessionData['modified']['content'] = (int)$this->getContentObject()->data['tstamp'];
            unset($result);
        }

        if ($result && !array_key_exists('modified', $sessionData)
          || !array_key_exists($this->getLang(), $sessionData['modified'])
          || $sessionData['modified'][$this->getLang()] < $this->getLastModified('page')) {
            $sessionData['modified'][$this->getLang()] = $this->getLastModified('page');
            unset($result);
        }

        if ($GLOBALS['TSFE']->sys_language_uid > 0) {
            if ($result && !array_key_exists('modified', $sessionData)
              || !array_key_exists($this->getLang(), $sessionData['modified'])
              || $sessionData['modified'][$this->getLang()] < $this->getLastModified('pageAlt')) {
                $sessionData['modified'][$this->getLang()] = $this->getLastModified('pageAlt');
                unset($result);
            }
        }

        $filterHash = md5(serialize($this->getFilter()));
        if ($result && !array_key_exists('filterHash', $sessionData) || $sessionData['filterHash'] != $filterHash) {
            $sessionData['filterHash'] = $filterHash;
            unset($result);
        }

        if (!$result) {
            $cacheKey = 'uids_' . $this->getContentKey() . $GLOBALS['TSFE']->sys_language_uid . '_' . $filterHash;

            if (!$this->getCache()->hasCache($cacheKey) || ($this->getCache()->hasCache($cacheKey)
              && $this->getCache()->getModified($cacheKey) < $this->getLastModified('page'))) {
                switch ($this->getSetting('displayType')) {
                    case 'selectedpages':
                        $result = $this->getSelectedPages();
                        break;
                    case 'allpages':
                        $result = $this->getAllPages();
                        break;
                    case 'teaserpages':
                        $result = $this->getTeaserPages();
                        break;
                    case 'selectedteaserpages':
                        $result = $this->getSelectedTeaserPages();
                        break;
                    case 'subpages':
                    case 'subpagesrecursive':
                    case 'selectedsubpages':
                    case 'selectedsubpagesrecursive':
                    default:
                        $result = $this->getSubPages(
                            $this->getPageUidsForSubPages($this->getSetting('displayType'))
                        );
                        break;
                }

                $this->getCache()->writeCache($cacheKey, $result);
                $sessionData['results'][$this->getLang()] = $result;
                $this->setSessionData($sessionData);
            } else {
                $result = $this->getCache()->loadCache($cacheKey);
            }

            if (!$result) {
                $result = array();
            }
        }

        return $result;
    }

    /**
     * @param array $result
     * @return \TYPO3\CMS\Extbase\Persistence\QueryResultInterface
     */
    public function getPages($result)
    {
        $excludeUids = array();
        if ($this->getSetting('excludeUids')) {
            $excludeUids = array_map('intval', explode(',', $this->getSetting('excludeUids')));
        }
        
        $pages = $this->pageRepository->findByUids(
            $result,
            $this->getSetting('date'),
            $excludeUids,
            $this->getLimit(),
            $this->getOffset(),
            array($this->getSorting(), $this->getSetting('sortingDirection'))
        );

        if ($this->getSetting('displayType') == 'selectedpages' && $this->getSetting('sorting') == 'selected') {
            $pageOrder = array();
            $pageCount = 0;
            foreach (explode(',', $this->getSetting('pages')) as $pageUid) {
                $pageOrder[(int)$pageUid] = ++$pageCount;
            }

            $orderedPages = array();
            foreach ($pages as $page) {
                $orderedPages[$pageOrder[$page->getUid()]] = $page;
            }

            ksort($orderedPages);
            $pages = $orderedPages;
        }

        return $pages;
    }

    /**
     * @return string
     */
    public function getSorting()
    {
        $sorting = $this->getSetting('sorting');
        if ($sorting == 'selected') {
            $sorting = 'sorting';
        }

        return $sorting;
    }

    /**
     * @param array $result
     * @return integer
     */
    public function getPagesCount($result)
    {
        $excludeUids = array();
        if ($this->getSetting('excludeUids')) {
            $excludeUids = array_map('intval', explode(',', $this->getSetting('excludeUids')));
        }

        $pagesCount = $this->pageRepository->countByUids(
            $result,
            $this->settings['date'],
            $excludeUids
        );

        return ceil($pagesCount / $this->getLimit());
    }

    /**
     * @param string $orderDirection
     * @return \IMIA\ImiaPageteaser\Domain\Model\Page
     */
    public function getFirstPage($orderDirection = 'asc')
    {
        $cacheKey = 'firstlast_' . $this->getContentKey() .
            $GLOBALS['TSFE']->sys_language_uid . '_' . $orderDirection;

        if (!$this->getCache()->hasCache($cacheKey) || ($this->getCache()->hasCache($cacheKey)
          && $this->getCache()->getModified($cacheKey) < $this->getLastModified('page'))) {
            switch ($this->getSetting('displayType')) {
                case 'selectedpages':
                    $result = $this->getSelectedPages(true, $orderDirection);
                    break;
                case 'allpages':
                    $result = $this->getAllPages(true, $orderDirection);
                    break;
                case 'teaserpages':
                    $result = $this->getTeaserPages(true, $orderDirection);
                    break;
                case 'selectedteaserpages':
                    $result = $this->getSelectedTeaserPages(true, $orderDirection);
                    break;
                case 'subpages':
                case 'subpagesrecursive':
                case 'selectedsubpages':
                case 'selectedsubpagesrecursive':
                default:
                    $result = $this->getSubPages(
                        $this->getPageUidsForSubPages($this->getSetting('displayType')),
                        true, $orderDirection
                    );
                    break;
            }
            if ($result) {
                $page = $result->toArray();
            }

            $this->getCache()->writeCache($cacheKey, $page);
        } else {
            $page = $this->getCache()->loadCache($cacheKey);
        }

        return $page;
    }

    /**
     * @return \IMIA\ImiaPageteaser\Domain\Model\Page
     */
    public function getLastPage()
    {
        return $this->getFirstPage('desc');
    }

    /**
     * @return array
     */
    public function getContentData()
    {
        if (!$this->contentData) {
            if ($this->allowParamOverwrite && GeneralUtility::_GP('contentUid')) {
                $this->contentData = array_shift($GLOBALS['TYPO3_DB']->exec_SELECTgetRows(
                    '*', 'tt_content', 'uid = ' . (int)GeneralUtility::_GP('contentUid')));
            } else {
                $this->contentData = $this->configurationManager->getContentObject()->data;
            }
        }

        return $this->contentData;
    }

    /**
     * @param boolean $onlyContent
     * @return integer
     */
    public function getContentUid($onlyContent = false)
    {
        $contentData = $this->getContentData();
        $contentUid = $contentData['uid'];
        if ($onlyContent && $contentData['uid'] == $GLOBALS['TSFE']->id && array_key_exists('TSconfig', $contentData)) {
            $contentUid = null;
        }

        return $contentUid;
    }

    /**
     * @return integer
     */
    public function getPage()
    {
        return $this->pageRepository->findOneByUid($this->getPageUid());
    }

    /**
     * @return integer
     */
    public function getPageUid()
    {
        if (!$this->pageUid) {
            if ($this->allowParamOverwrite && GeneralUtility::_GP('backPid')) {
                $this->pageUid = (int)GeneralUtility::_GP('backPid');
            } else {
                $this->pageUid = $GLOBALS['TSFE']->id;
            }
        }

        return $this->pageUid;
    }

    /**
     * @return array
     */
    public function getFilterOptions()
    {
        $firstPage = $this->getFirstPage();
        $lastPage = $this->getLastPage();

        $filterOptions = array(
            'month' => array(0 => $this->translate('filter_choose')),
            'year' => array(0 => $this->translate('filter_choose')),
            'month-year'=> array(0 => $this->translate('filter_choose')),
        );

        if ($firstPage) {
            for ($month = 1; $month <= 12; $month++) {
                $filterOptions['month'][$month] = $this->translate('filter_month_' . $month);
            }

            if ($firstPage['lastUpdated']) {
                if ($this->getSetting('date') == 'past') {
                    for ($year = (int)date('Y', $firstPage['lastUpdated']); $year <= date('Y'); $year++) {
                        $filterOptions['year'][$year] = $year;
                    }
                } elseif ($lastPage) {
                    for ($year = (int)date('Y', $firstPage['lastUpdated']); $year <= (int)date('Y', $lastPage['lastUpdated']); $year++) {
                        $filterOptions['year'][$year] = $year;
                    }
                } else {
                    for ($year = (int)date('Y', $firstPage['lastUpdated']); $year <= date('Y'); $year++) {
                        $filterOptions['year'][$year] = $year;
                    }
                }
            }
        }

        return $filterOptions;
    }

    /**
     * @param string $orderDirection
     * @return string
     */
    public function getRealOrderDirection($orderDirection)
    {
        if (strtolower($this->getSetting('sortingDirection')) == 'asc') {
            if (strtolower($orderDirection) == 'asc') {
                $realOrderDirection = 'asc';
            } else {
                $realOrderDirection = 'desc';
            }
        } else {
            if (strtolower($orderDirection) == 'asc') {
                $realOrderDirection = 'desc';
            } else {
                $realOrderDirection = 'asc';
            }
        }

        return $realOrderDirection;
    }

    /**
     * @param string $type
     * @return object
     */
    public function getLastModified($type)
    {
        if (!array_key_exists($type, self::$lastModified)) {
            switch ($type) {
                case 'pageAlt':
                    self::$lastModified[$type] = $this->pageAltRepository->findLastModified('pages_language_overlay');
                    break;
                case 'page':
                default:
                    self::$lastModified[$type] = $this->pageRepository->findLastModified();
                    break;
            }
        }

        return self::$lastModified[$type];
    }

    /**
     * @return array
     */
    public function getDoktypes()
    {
        if (!$this->doktypes) {
            $this->doktypes = array();
            foreach (explode(',', $this->getSetting('doktypes')) as $doktype) {
                $this->doktypes[] = (int)$doktype;

                switch ((int)$doktype) {
                    case 1:
                        $this->doktypes[] = 3;
                        $this->doktypes[] = 4;
                        break;
                    case 100:
                        $this->doktypes[] = 103;
                        $this->doktypes[] = 104;
                        $this->doktypes[] = 106;
                        break;
                    case 110:
                        $this->doktypes[] = 111;
                        $this->doktypes[] = 112;
                        $this->doktypes[] = 113;
                        $this->doktypes[] = 114;
                        $this->doktypes[] = 115;
                        $this->doktypes[] = 116;
                        break;
                    case 120:
                        $this->doktypes[] = 123;
                        $this->doktypes[] = 124;
                        $this->doktypes[] = 126;
                        break;
                    case 130:
                        $this->doktypes[] = 133;
                        $this->doktypes[] = 134;
                        $this->doktypes[] = 136;
                        break;
                    case 140:
                        $this->doktypes[] = 143;
                        $this->doktypes[] = 144;
                        $this->doktypes[] = 146;
                        break;
                }
            }
        }

        return $this->doktypes;
    }

    /**
     * @return array
     */
    public function getFilter()
    {
        if (!$this->filter) {
            $this->filter = array();

            if (!$this->getSetting('disablePersistentFilter')) {
                $sessionData = $this->getSessionData();

                if (array_key_exists('filter', $sessionData) && $sessionData['filter']) {
                    $this->filter = $sessionData['filter'];
                }
            }

            if ($this->request->hasArgument('filter') && $this->request->getArgument('filter')) {
                $this->filter = (array)$this->request->getArgument('filter');
            }

            if (!$this->getSetting('disablePersistentFilter')) {
                $sessionData['filter'] = $this->filter;
                $this->setSessionData($sessionData);
            }

            // Hook begin
            foreach ($this->getHookObjects() as $hook) {
                if (method_exists($hook, 'getFilter')) {
                    $hook->getFilter($this->filter, $this->settings, $this);
                }
            }

            if (!$this->getSetting('disablePersistentFilter')) {
                if (serialize($sessionData['filter']) != serialize($this->filter)) {
                    $sessionData = $this->getSessionData();
                    $sessionData['filter'] = $this->filter;
                    $this->setSessionData($sessionData);
                }
            }
            // Hook end
        }

        return $this->filter;
    }

    /**
     * @return integer
     */
    public function getPageNum()
    {
        if (!$this->pageNum) {
            $this->pageNum = 1;
            if ($this->request->hasArgument('page') && $this->request->getArgument('page')) {
                $this->pageNum = (int)$this->request->getArgument('page');
            }
        }

        return $this->pageNum;
    }

    /**
     * @return integer
     */
    public function getLimit()
    {
        if (!$this->limit) {
            $this->limit = (int)$this->getSetting('limit');

            foreach ($this->getHookObjects() as $hook) {
                if (method_exists($hook, 'getLimit')) {
                    $hook->getLimit($this->limit, $this->getFilter(), $this);
                }
            }
        }

        return $this->limit;
    }

    /**
     * @return integer
     */
    public function getOffset()
    {
        $offset = (int)$this->getSetting('offset');

        return $this->getPageNum() * $this->getLimit() - $this->getLimit() + $offset;
    }

    /**
     * @return array
     */
    public function getCategories()
    {
        if (!$this->categories) {
            $contentUid = $this->getContentUid(true);
            if (!$contentUid) {
                if ($this->getSetting('categories')) {
                    $this->categories = $this->categoryRepository->findByUids(
                        array_map('intval', explode(',', $this->getSetting('categories')))
                    );
                } else {
                    $this->categories = array();
                }
            } else {
                $this->categories = Helper::getRecordCategories(
                    $contentUid,
                    'tt_content',
                    'categories',
                    $this->categoryRepository
                );
            }
        }

        return $this->categories;
    }

    /**
     * @return array
     */
    public function getSessionData()
    {
        if (!$this->sessionData) {
            $this->sessionData = $this->sessionStorage->get($this->getSessionDataKey());

            if (!$this->sessionData) {
                $this->sessionData = array();
            }
        }

        return $this->sessionData;
    }

    /**
     * @param array $sessionData
     */
    public function setSessionData($sessionData)
    {
        $this->sessionData = $sessionData;
        $this->sessionStorage->set($this->getSessionDataKey(), $this->sessionData);
    }

    /**
     * @return integer
     */
    public function getLang()
    {
        return $GLOBALS['TSFE']->sys_language_uid;
    }

    /**
     * @return Cache
     */
    public function getCache()
    {
        if (!self::$cache) {
            self::$cache = new Cache(PATH_site . 'typo3temp/Cache/Code/cache_core/imia_pageteaser/');
        }

        return self::$cache;
    }

    /*
     * @param boolean $getFirst
     * @param string $orderDirection
     * @return mixed
     */
    protected function getSelectedPages($getFirst = false, $orderDirection = null)
    {
        if (!$getFirst) {
            $result = $this->pageRepository->findUidsByUidsAndDoktypesAndCategories(
                array_map('intval', explode(',', $this->getSetting('pages'))),
                $this->getDoktypes(),
                $this->getCategories(),
                $this->getFilter(),
                array($this->getSorting(), $this->getRealOrderDirection($orderDirection))
            );
        } else {
            $result = array_shift($this->pageRepository->findByUidsAndDoktypesAndCategories(
                array_map('intval', explode(',', $this->getSetting('pages'))),
                $this->getDoktypes(),
                $this->getCategories(),
                null,
                1,
                0,
                array('lastUpdated', $orderDirection)
            ));
        }

        return $result;
    }

    /**
     * @param boolean $getFirst
     * @param string $orderDirection
     * @return mixed
     */
    protected function getAllPages($getFirst = false, $orderDirection = 'asc')
    {
        if (!$getFirst) {
            $result = $this->pageRepository->findUidsByDoktypesAndCategories(
                $this->getDoktypes(),
                $this->getCategories(),
                $this->getFilter(),
                array($this->getSorting(), $this->getRealOrderDirection($orderDirection))
            );
        } else {
            $result = array_shift($this->pageRepository->findByDoktypesAndCategories(
                $this->getDoktypes(),
                $this->getCategories(),
                null,
                1,
                0,
                array('lastUpdated', $orderDirection)
            ));
        }

        return $result;
    }

    /**
     * @param boolean $getFirst
     * @param string $orderDirection
     * @return mixed
     */
    protected function getTeaserPages($getFirst = false, $orderDirection = 'asc')
    {
        if (!$getFirst) {
            $result = $this->pageRepository->findTeaserPagesByUids(
                array($this->getPageUid())
            );
        } else {
            $pageUids = $this->pageRepository->findTeaserPagesByUids(
                array_map('intval', explode(',', $this->getSetting('pages')))
            );

            if (strtolower($orderDirection) == 'desc') {
                $pageUid = array_pop($pageUids);
            } else {
                $pageUid = array_shift($pageUids);
            }

            $result = $this->pageRepository->findOneByUid($pageUid);
        }

        return $result;
    }

    /**
     * @param boolean $getFirst
     * @param string $orderDirection
     * @return mixed
     */
    protected function getSelectedTeaserPages($getFirst = false, $orderDirection = null)
    {
        if (!$getFirst) {
            $result = $this->pageRepository->findTeaserPagesByUids(
                array_map('intval', explode(',', $this->getSetting('pages')))
            );
        } else {
            $pageUids = $this->pageRepository->findTeaserPagesByUids(
                array_map('intval', explode(',', $this->getSetting('pages')))
            );

            if (strtolower($orderDirection) == 'desc') {
                $pageUid = array_pop($pageUids);
            } else {
                $pageUid = array_shift($pageUids);
            }

            $result = $this->pageRepository->findOneByUid($pageUid);
        }

        return $result;
    }

    /**
     * @param array $pageUids
     * @param boolean $getFirst
     * @param string $orderDirection
     * @return mixed
     */
    protected function getSubPages($pageUids, $getFirst = false, $orderDirection = null)
    {
        if (!$getFirst) {
            $result = $this->pageRepository->findUidsByPidsAndDoktypesAndCategories(
                $pageUids,
                $this->getDoktypes(),
                $this->getCategories(),
                $this->getFilter(),
                array($this->getSorting(), $this->getRealOrderDirection($orderDirection))
            );
        } else {
            $result = array_shift($this->pageRepository->findByPidsAndDoktypesAndCategories(
                $pageUids,
                $this->getDoktypes(),
                $this->getCategories(),
                null,
                1,
                0,
                array('lastUpdated', strtolower($orderDirection))
            ));
        }

        return $result;
    }

    /**
     *
     * @param string $displayType
     * @return array
     */
    protected function getPageUidsForSubPages($displayType)
    {
        $cacheKey = 'storage-pids_' . $this->getContentKey();
        if (!$this->getCache()->hasCache($cacheKey) || ($this->getCache()->hasCache($cacheKey)
                && $this->getCache()->getModified($cacheKey) < $this->getLastModified('page'))) {
            $recursive = false;
            switch ($displayType) {
                case 'selectedsubpagesrecursive':
                    $recursive = true;
                case 'selectedsubpages':
                    $pageUids = array_map('intval', explode(',', $this->getSetting('pages')));
                    break;
                case 'subpagesrecursive':
                    $recursive = true;
                case 'subpages':
                default:
                    $pageUids = array($this->getPageUid());
                    break;
            }

            if ($recursive) {
                $allPageUids = array();
                foreach ($pageUids as $pageUid) {
                    $allPageUids = array_merge($allPageUids, $this->getSubPageUidsForPageUidRecursive($pageUid));
                }

                $pageUids = array_unique($allPageUids);
                $this->getCache()->writeCache($cacheKey, $pageUids);
            }

            if (count($pageUids) == 0) {
                $pageUids[] = 0;
            }
        } else {
            $pageUids = $this->getCache()->loadCache($cacheKey);
        }

        return $pageUids;
    }

    /**
     * @param integer $pageUid
     * @return array
     */
    protected function getSubPageUidsForPageUidRecursive($pageUid)
    {
        $pageUids = array();
        $page = $GLOBALS['TYPO3_DB']->exec_SELECTgetSingleRow('uid, doktype, mount_pid', 'pages',
            'uid = ' . (int)$pageUid . $GLOBALS['TSFE']->sys_page->enableFields('pages'));

        if ($page) {
            if ($page['doktype'] == 7 && $page['mount_pid']) {
                $pageUid = (int)$page['mount_pid'];
            }

            // recursive Query with DB-object instead of extbase for better performance
            $res = $GLOBALS['TYPO3_DB']->exec_SELECTquery('uid', 'pages', 'pid = ' . (int)$pageUid . $GLOBALS['TSFE']->sys_page->enableFields('pages'));
            while ($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($res)) {
                $pageUids = array_merge($pageUids, $this->getSubPageUidsForPageUidRecursive((int)$row['uid']));
            }

            $pageUids[] = $pageUid;
        }

        return $pageUids;
    }

    /**
     * @return string
     */
    protected function getSessionDataKey()
    {
        if (!$this->sessionDataKey) {
            $this->sessionDataKey = 'pt_' . $this->getPageUid() . '_' . $this->getContentKey();

            foreach ($this->getHookObjects() as $hook) {
                if (method_exists($hook, 'getSessionDataKey')) {
                    $hook->getSessionDataKey($this->sessionDataKey, $this->getPageUid(), $this->getContentData(), $this->settings, $this);
                }
            }
        }

        return $this->sessionDataKey;
    }

    /**
     * @return string
     */
    protected function getContentKey()
    {
        return $this->getContentUid(true) ?: md5(serialize($this->getContentData()) . serialize($this->getSettings()));
    }

    /**
     * Overwrite settings, if alternative contentUid is defined
     */
    protected function updateSettings()
    {
        $contentData = $this->getContentData();
        if ($contentData['uid'] != $this->configurationManager->getContentObject()->data['uid']) {
            $flexFormService = $this->objectManager->get('TYPO3\CMS\Extbase\Service\FlexFormService');
            $flexFormData = $flexFormService->convertFlexFormContentToArray($contentData['pi_flexform']);
            if ($flexFormData && array_key_exists('settings', $flexFormData) && is_array($flexFormData['settings'])) {
                foreach ($flexFormData['settings'] as $key => $val) {
                    $this->settings[$key] = $val;
                }
            }
        }
    }
}