<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2013-2014 IMIA net based solutions (d.frerich@imia.de)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

namespace IMIA\ImiaPageteaser\Domain\Model;

use TYPO3\CMS\Extbase\Persistence\ObjectStorage;
use IMIA\ImiaBaseExt\Annotation\SQL;
use IMIA\ImiaBaseExt\Annotation\TCA;

/**
 * @package     imia_pageteaser
 * @subpackage  Domain\Model
 * @author      David Frerich <d.frerich@imia.de>
 *
 * @TCA\Table(
 *  types = {
 *      100: "--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.standard;standard, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.title;title, tx_templavoila_flex;;;;1-1-1, --div--;LLL::page_tab_content,--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.abstract;abstract, teaser_image, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.editorial;editorial, --palette--;LLL:EXT:imiapageteaser/Resources/Private/Language/DB.xlf:palettes_archiving;archiving, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.visibility;visibility, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.access;access, categories,  --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.metadata, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.metatags;metatags, --div--;LLL:EXT:cms/locallang_tca.xlf:pages.tabs.appearance, --palette--;LLL:EXT:cms/locallang_tca.xlf:pages.palettes.layout;layout,	--palette--;LLL:EXT:cms/locallang_tca.xlf:pages.palettes.replace;replace, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.behaviour, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.links;links, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.caching;caching, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.language;language, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.resources, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.media;media",
 *      103: "--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.external;external, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.title;title, tx_templavoila_flex;;;;1-1-1, --div--;LLL::page_tab_content,--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.abstract;abstract, teaser_image, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.editorial;editorial, --palette--;LLL:EXT:imiapageteaser/Resources/Private/Language/DB.xlf:palettes_archiving;archiving, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.visibility;visibility, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.access;access, categories,  --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.behaviour, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.links;links, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.caching;caching, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.language;language, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.resources, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.media;media",
 *      104: "--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.shortcut;shortcut, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.shortcutpage;shortcutpage, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.title;title, tx_templavoila_flex;;;;1-1-1, --div--;LLL::page_tab_content,--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.abstract;abstract, teaser_image, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.editorial;editorial, --palette--;LLL:EXT:imiapageteaser/Resources/Private/Language/DB.xlf:palettes_archiving;archiving, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.visibility;visibility, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.access;access, categories,  --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.behaviour, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.links;links, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.caching;caching, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.language;language, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.resources, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.media;media",
 *      106: "--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.standard;filepalette, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.title;title, tx_templavoila_flex;;;;1-1-1, --div--;LLL::page_tab_content,--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.abstract;abstract, teaser_image, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.editorial;editorial, --palette--;LLL:EXT:imiapageteaser/Resources/Private/Language/DB.xlf:palettes_archiving;archiving, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.visibility;visibility, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.access;access, categories,  --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.behaviour, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.links;links, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.caching;caching, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.language;language, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.resources, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.media;media",
 *      110: "--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.standard;standard, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.title;title, tx_templavoila_flex;;;;1-1-1, --div--;LLL::page_tab_content,--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.abstract;abstract, teaser_image, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.editorial;editorial, --palette--;LLL:EXT:imiapageteaser/Resources/Private/Language/DB.xlf:palettes_archiving;archiving, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.visibility;visibility, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.access;access, categories,  --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.metadata, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.metatags;metatags, --div--;LLL:EXT:cms/locallang_tca.xlf:pages.tabs.appearance, --palette--;LLL:EXT:cms/locallang_tca.xlf:pages.palettes.layout;layout,--palette--;LLL:EXT:cms/locallang_tca.xlf:pages.palettes.replace;replace, --div--;LLL::page_tab_content,--div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.behaviour, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.links;links, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.caching;caching, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.language;language, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.resources, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.media;media",
 *      111: "--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.standard;standard, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.title;title, tx_templavoila_flex;;;;1-1-1, --div--;LLL::page_tab_content,--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.abstract;abstract, teaser_image, gallery_images, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.editorial;editorial, --palette--;LLL:EXT:imiapageteaser/Resources/Private/Language/DB.xlf:palettes_archiving;archiving, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.visibility;visibility, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.access;access, categories,  --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.behaviour, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.caching;caching, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.language;language, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.resources, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.media;media",
 *      112: "--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.external;external, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.title;title, tx_templavoila_flex;;;;1-1-1, --div--;LLL::page_tab_content,--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.abstract;abstract, teaser_image, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.editorial;editorial, --palette--;LLL:EXT:imiapageteaser/Resources/Private/Language/DB.xlf:palettes_archiving;archiving, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.visibility;visibility, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.access;access, categories,  --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.behaviour, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.caching;caching, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.language;language, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.resources, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.media;media",
 *      113: "--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.external;external, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.title;title, tx_templavoila_flex;;;;1-1-1, --div--;LLL::page_tab_content,--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.abstract;abstract, teaser_image, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.editorial;editorial, --palette--;LLL:EXT:imiapageteaser/Resources/Private/Language/DB.xlf:palettes_archiving;archiving, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.visibility;visibility, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.access;access, categories,  --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.behaviour, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.links;links, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.caching;caching, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.language;language, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.resources, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.media;media",
 *      114: "--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.shortcut;shortcut, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.shortcutpage;shortcutpage, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.title;title, tx_templavoila_flex;;;;1-1-1, --div--;LLL::page_tab_content,--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.abstract;abstract, teaser_image, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.editorial;editorial, --palette--;LLL:EXT:imiapageteaser/Resources/Private/Language/DB.xlf:palettes_archiving;archiving, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.visibility;visibility, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.access;access, categories,  --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.behaviour, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.links;links, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.caching;caching, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.language;language, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.resources, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.media;media",
 *      115: "--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.standard;standard, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.title;title, tx_templavoila_flex;;;;1-1-1, --div--;LLL::page_tab_content,--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.abstract;abstract, teaser_image, gallery_pdf, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.editorial;editorial, --palette--;LLL:EXT:imiapageteaser/Resources/Private/Language/DB.xlf:palettes_archiving;archiving, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.visibility;visibility, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.access;access, categories,  --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.behaviour, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.caching;caching, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.language;language, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.resources, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.media;media",
 *      116: "--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.standard;filepalette, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.title;title, tx_templavoila_flex;;;;1-1-1, --div--;LLL::page_tab_content,--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.abstract;abstract, teaser_image, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.editorial;editorial, --palette--;LLL:EXT:imiapageteaser/Resources/Private/Language/DB.xlf:palettes_archiving;archiving, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.visibility;visibility, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.access;access, categories,  --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.behaviour, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.links;links, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.caching;caching, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.language;language, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.resources, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.media;media",
 *      120: "--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.standard;standard, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.title;title, tx_templavoila_flex;;;;1-1-1, --div--;LLL::page_tab_content,--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.abstract;abstract, teaser_image, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.editorial;editorial, --palette--;LLL:EXT:imiapageteaser/Resources/Private/Language/DB.xlf:palettes_archiving;archiving, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.visibility;visibility, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.access;access, categories,  --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.metadata, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.metatags;metatags, --div--;LLL:EXT:cms/locallang_tca.xlf:pages.tabs.appearance, --palette--;LLL:EXT:cms/locallang_tca.xlf:pages.palettes.layout;layout,--palette--;LLL:EXT:cms/locallang_tca.xlf:pages.palettes.replace;replace, --div--;LLL::page_tab_content,--div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.behaviour, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.links;links, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.caching;caching, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.language;language, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.resources, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.media;media",
 *      123: "--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.external;external, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.title;title, tx_templavoila_flex;;;;1-1-1, --div--;LLL::page_tab_content,--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.abstract;abstract, teaser_image, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.editorial;editorial, --palette--;LLL:EXT:imiapageteaser/Resources/Private/Language/DB.xlf:palettes_archiving;archiving, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.visibility;visibility, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.access;access, categories,  --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.behaviour, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.links;links, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.caching;caching, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.language;language, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.resources, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.media;media",
 *      124: "--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.shortcut;shortcut, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.shortcutpage;shortcutpage, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.title;title, tx_templavoila_flex;;;;1-1-1, --div--;LLL::page_tab_content,--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.abstract;abstract, teaser_image, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.editorial;editorial, --palette--;LLL:EXT:imiapageteaser/Resources/Private/Language/DB.xlf:palettes_archiving;archiving, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.visibility;visibility, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.access;access, categories,  --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.behaviour, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.links;links, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.caching;caching, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.language;language, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.resources, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.media;media",
 *      126: "--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.standard;filepalette, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.title;title, tx_templavoila_flex;;;;1-1-1, --div--;LLL::page_tab_content,--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.abstract;abstract, teaser_image, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.editorial;editorial, --palette--;LLL:EXT:imiapageteaser/Resources/Private/Language/DB.xlf:palettes_archiving;archiving, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.visibility;visibility, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.access;access, categories,  --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.behaviour, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.links;links, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.caching;caching, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.language;language, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.resources, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.media;media",
 *      130: "--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.standard;standard, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.title;title, tx_templavoila_flex;;;;1-1-1, --div--;LLL::page_tab_content,--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.abstract;abstract, teaser_image, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.editorial;editorial, --palette--;LLL:EXT:imiapageteaser/Resources/Private/Language/DB.xlf:palettes_archiving;archiving, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.visibility;visibility, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.access;access, categories,  --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.metadata, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.metatags;metatags, --div--;LLL:EXT:cms/locallang_tca.xlf:pages.tabs.appearance, --palette--;LLL:EXT:cms/locallang_tca.xlf:pages.palettes.layout;layout,--palette--;LLL:EXT:cms/locallang_tca.xlf:pages.palettes.replace;replace, --div--;LLL::page_tab_content,--div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.behaviour, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.links;links, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.caching;caching, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.language;language, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.resources, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.media;media",
 *      133: "--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.external;external, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.title;title, tx_templavoila_flex;;;;1-1-1, --div--;LLL::page_tab_content,--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.abstract;abstract, teaser_image, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.editorial;editorial, --palette--;LLL:EXT:imiapageteaser/Resources/Private/Language/DB.xlf:palettes_archiving;archiving, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.visibility;visibility, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.access;access, categories,  --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.behaviour, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.links;links, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.caching;caching, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.language;language, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.resources, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.media;media",
 *      134: "--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.shortcut;shortcut, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.shortcutpage;shortcutpage, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.title;title, tx_templavoila_flex;;;;1-1-1, --div--;LLL::page_tab_content,--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.abstract;abstract, teaser_image, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.editorial;editorial, --palette--;LLL:EXT:imiapageteaser/Resources/Private/Language/DB.xlf:palettes_archiving;archiving, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.visibility;visibility, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.access;access, categories,  --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.behaviour, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.links;links, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.caching;caching, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.language;language, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.resources, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.media;media",
 *      136: "--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.standard;filepalette, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.title;title, tx_templavoila_flex;;;;1-1-1, --div--;LLL::page_tab_content,--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.abstract;abstract, teaser_image, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.editorial;editorial, --palette--;LLL:EXT:imiapageteaser/Resources/Private/Language/DB.xlf:palettes_archiving;archiving, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.visibility;visibility, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.access;access, categories,  --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.behaviour, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.links;links, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.caching;caching, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.language;language, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.resources, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.media;media",
 *      140: "--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.standard;standard, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.title;title, tx_templavoila_flex;;;;1-1-1, --div--;LLL::page_tab_content,--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.abstract;abstract, teaser_image, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.editorial;editorial, --palette--;LLL:EXT:imiapageteaser/Resources/Private/Language/DB.xlf:palettes_archiving;archiving, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.visibility;visibility, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.access;access, categories,  --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.metadata, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.metatags;metatags, --div--;LLL:EXT:cms/locallang_tca.xlf:pages.tabs.appearance, --palette--;LLL:EXT:cms/locallang_tca.xlf:pages.palettes.layout;layout,--palette--;LLL:EXT:cms/locallang_tca.xlf:pages.palettes.replace;replace, --div--;LLL::page_tab_content,--div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.behaviour, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.links;links, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.caching;caching, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.language;language, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.resources, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.media;media",
 *      143: "--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.external;external, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.title;title, tx_templavoila_flex;;;;1-1-1, --div--;LLL::page_tab_content,--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.abstract;abstract, teaser_image, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.editorial;editorial, --palette--;LLL:EXT:imiapageteaser/Resources/Private/Language/DB.xlf:palettes_archiving;archiving, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.visibility;visibility, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.access;access, categories,  --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.behaviour, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.links;links, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.caching;caching, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.language;language, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.resources, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.media;media",
 *      144: "--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.shortcut;shortcut, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.shortcutpage;shortcutpage, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.title;title, tx_templavoila_flex;;;;1-1-1, --div--;LLL::page_tab_content,--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.abstract;abstract, teaser_image, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.editorial;editorial, --palette--;LLL:EXT:imiapageteaser/Resources/Private/Language/DB.xlf:palettes_archiving;archiving, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.visibility;visibility, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.access;access, categories,  --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.behaviour, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.links;links, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.caching;caching, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.language;language, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.resources, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.media;media",
 *      146: "--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.standard;filepalette, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.title;title, tx_templavoila_flex;;;;1-1-1, --div--;LLL::page_tab_content,--palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.abstract;abstract, teaser_image, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.editorial;editorial, --palette--;LLL:EXT:imiapageteaser/Resources/Private/Language/DB.xlf:palettes_archiving;archiving, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.visibility;visibility, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.access;access, categories,  --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.behaviour, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.links;links, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.caching;caching, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.language;language, --div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.resources, --palette--;LLL:EXT:cms/locallang_tca.xml:pages.palettes.media;media"
 *  },
 *  palettes = {
 *      "standard": "doktype;LLL:EXT:cms/locallang_tca.xlf:pages.doktype_formlabel, create_language",
 *      "shortcut": "doktype;LLL:EXT:cms/locallang_tca.xlf:pages.doktype_formlabel, shortcut_mode;LLL:EXT:cms/locallang_tca.xlf:pages.shortcut_mode_formlabel, create_language",
 *      "mountpoint": "doktype;LLL:EXT:cms/locallang_tca.xlf:pages.doktype_formlabel, mount_pid_ol;LLL:EXT:cms/locallang_tca.xlf:pages.mount_pid_ol_formlabel, create_language",
 *      "external": "doktype;LLL:EXT:cms/locallang_tca.xlf:pages.doktype_formlabel, urltype;LLL:EXT:cms/locallang_tca.xlf:pages.urltype_formlabel, url;LLL:EXT:cms/locallang_tca.xlf:pages.url_formlabel, create_language",
 *      "filepalette": "doktype;LLL:EXT:cms/locallang_tca.xlf:pages.doktype_formlabel, create_language, --linebreak--, file",
 *      "archiving": "archivedate",
 *  }
 * )
 */
class Page extends \IMIA\ImiaBaseExt\Domain\Model\Page
{
    /**
     * @SQL\Column(name="pid", create=false)
     * @TCA\Passthrough
     * @var integer
     */
    protected $pid;

    /**
     * @var \TYPO3\CMS\Core\Resource\FileRepository
     * @inject
     * @FIXME How to do this a better way?
     */
    protected $fileRepository;

    /**
     * @var IMIA\ImiaPageteaser\Domain\Repository\PageRepository
     * @inject
     * @FIXME How to do this a better way to sort the relation-items with a 1:n field?
     */
    protected $pageRepository;

    /**
     * @var IMIA\ImiaPageteaser\Domain\Repository\PageAltRepository
     * @inject
     * @FIXME How to do this a better way to sort the relation-items with a 1:n field?
     */
    protected $pageAltRepository;

    /**
     * @var array
     */
    protected $translations;

    /**
     * @var array
     */
    protected $teaserPageObjects;

    /**
     * @var integer
     */
    protected $contentFileRelations;

    /**
     * @var \IMIA\ImiaPageteaser\Domain\Model\Page
     * @lazy
     */
    protected $shortcut;

    /**
     * @SQL\Column(name="cruser_id", create=false)
     * @TCA\Select(
     *  foreign_table = "be_users",
     *  foreign_table_where = " AND be_users.username NOT LIKE '\_%'"
     * )
     *
     * @var \IMIA\ImiaPageteaser\Domain\Model\BackendUser
     * @lazy
     */
    protected $cruser;

    /**
     * @var \IMIA\ImiaPageteaser\Domain\Model\Page
     * @lazy
     */
    protected $contentFromPid;

    /**
     * @var boolean
     */
    protected $hidden;

    /**
     * @var \DateTime
     */
    protected $starttime;

    /**
     * @var \DateTime
     */
    protected $endtime;

    /**
     * @TCA\Select(
     *  items = {
     *      99: {"LLL::page_doktype_divider_news", "--div--"},
     *      100: {"LLL::page_doktype_news", 100, "EXTREL::Resources/Public/Images/Icons/PageDoktypeNews.png"},
     *      101: {"LLL::page_doktype_news_shortcut", 104, "EXTREL::Resources/Public/Images/Icons/PageDoktypeNewsShortcut.png"},
     *      102: {"LLL::page_doktype_news_external", 103, "EXTREL::Resources/Public/Images/Icons/PageDoktypeNewsExternal.png"},
     *      103: {"LLL::page_doktype_news_file", 106, "EXTREL::Resources/Public/Images/Icons/PageDoktypeNewsFile.png"},
     *      109: {"LLL::page_doktype_divider_gallery", "--div--"},
     *      110: {"LLL::page_doktype_gallery", 110, "EXTREL::Resources/Public/Images/Icons/PageDoktypeGallery.png"},
     *      112: {"LLL::page_doktype_gallery_shortcut", 114, "EXTREL::Resources/Public/Images/Icons/PageDoktypeGalleryShortcut.png"},
     *      113: {"LLL::page_doktype_gallery_external", 113, "EXTREL::Resources/Public/Images/Icons/PageDoktypeGalleryExternal.png"},
     *      114: {"LLL::page_doktype_gallery_images", 111, "EXTREL::Resources/Public/Images/Icons/PageDoktypeGalleryImages.png"},
     *      115: {"LLL::page_doktype_gallery_video", 112, "EXTREL::Resources/Public/Images/Icons/PageDoktypeGalleryVideo.png"},
     *      116: {"LLL::page_doktype_gallery_pdf", 115, "EXTREL::Resources/Public/Images/Icons/PageDoktypeGalleryPdf.png"},
     *      117: {"LLL::page_doktype_gallery_file", 116, "EXTREL::Resources/Public/Images/Icons/PageDoktypeGalleryFile.png"},
     *      119: {"LLL::page_doktype_divider_product", "--div--"},
     *      120: {"LLL::page_doktype_product", 120, "EXTREL::Resources/Public/Images/Icons/PageDoktypeProduct.png"},
     *      122: {"LLL::page_doktype_product_shortcut", 124, "EXTREL::Resources/Public/Images/Icons/PageDoktypeProductShortcut.png"},
     *      123: {"LLL::page_doktype_product_external", 123, "EXTREL::Resources/Public/Images/Icons/PageDoktypeProductExternal.png"},
     *      124: {"LLL::page_doktype_product_file", 126, "EXTREL::Resources/Public/Images/Icons/PageDoktypeProductFile.png"},
     *      129: {"LLL::page_doktype_divider_event", "--div--"},
     *      130: {"LLL::page_doktype_event", 130, "EXTREL::Resources/Public/Images/Icons/PageDoktypeEvent.png"},
     *      132: {"LLL::page_doktype_event_shortcut", 134, "EXTREL::Resources/Public/Images/Icons/PageDoktypeEventShortcut.png"},
     *      133: {"LLL::page_doktype_event_external", 133, "EXTREL::Resources/Public/Images/Icons/PageDoktypeEventExternal.png"},
     *      134: {"LLL::page_doktype_event_file", 136, "EXTREL::Resources/Public/Images/Icons/PageDoktypeEventFile.png"},
     *      139: {"LLL::page_doktype_divider_article", "--div--"},
     *      140: {"LLL::page_doktype_article", 140, "EXTREL::Resources/Public/Images/Icons/PageDoktypeArticle.png"},
     *      142: {"LLL::page_doktype_article_shortcut", 144, "EXTREL::Resources/Public/Images/Icons/PageDoktypeArticleShortcut.png"},
     *      143: {"LLL::page_doktype_article_external", 143, "EXTREL::Resources/Public/Images/Icons/PageDoktypeArticleExternal.png"},
     *      144: {"LLL::page_doktype_article_file", 146, "EXTREL::Resources/Public/Images/Icons/PageDoktypeArticleFile.png"},
     *  }
     * )
     *
     * @var integer
     */
    protected $doktype;

    /**
     * @SQL\Column(type="varchar", length=255, nullable=false)
     * @TCA\Group(
     *  internal_type = "db",
     *  allowed = "pages",
     *  size = 5,
     *  maxitems = 5,
     *  minitems = 0,
     *  show_thumbs = true
     * )
     * @TCA\AddToTypes(position="after:target")
     *
     * @var string
     */
    protected $teaserPages;

    /**
     * @var array
     */
    protected $media;

    /**
     * @SQL\Column(type="int", length=11, nullable=false)
     * @TCA\ImageFiles(
     *  maxitems = 1,
     *  foreign_types = {
     *      \TYPO3\CMS\Core\Resource\File::FILETYPE_IMAGE: {"showitem": "title,alternative;;;;3-3-3,--palette--;;filePalette" }
     *  }
     * )
     * @TCA\AddToTypes(position="after:target")
     *
     * @var integer
     */
    protected $teaserImage;

    /**
     * @SQL\Column(type="int", length=11, nullable=false)
     * @TCA\ImageFiles(
     *  foreign_types = {
     *      \TYPO3\CMS\Core\Resource\File::FILETYPE_IMAGE: {"showitem": "title,alternative;;;;3-3-3,description,--palette--;;filePalette" }
     *  }
     * )
     *
     * @var integer
     */
    protected $galleryImages;

    /**
     * @SQL\Column(type="int", length=11, nullable=false)
     * @TCA\Files(
     *  foreign_types = {
     *      \TYPO3\CMS\Core\Resource\File::FILETYPE_UNKNOWN: {"showitem": "--palette--;;filePalette" },
     *      \TYPO3\CMS\Core\Resource\File::FILETYPE_TEXT: {"showitem": "--palette--;;filePalette" },
     *      \TYPO3\CMS\Core\Resource\File::FILETYPE_IMAGE: {"showitem": "--palette--;;filePalette" },
     *      \TYPO3\CMS\Core\Resource\File::FILETYPE_AUDIO: {"showitem": "--palette--;;filePalette" },
     *      \TYPO3\CMS\Core\Resource\File::FILETYPE_VIDEO: {"showitem": "--palette--;;filePalette" },
     *      \TYPO3\CMS\Core\Resource\File::FILETYPE_APPLICATION: {"showitem": "--palette--;;filePalette" },
     *  },
     *  maxitems = 1,
     * )
     *
     * @var int
     */
    protected $file;

    /**
     * @SQL\Column(type="int", length=11)
     * @TCA\FilesPDF(
     *  maxitems = 1
     * )
     *
     * @var integer
     */
    protected $galleryPdf;

    /**
     * @SQL\Column(type="int", length=11, nullable=false)
     * @TCA\Categories(
     *  label = "LLL::page_categories",
     *  size = 5,
     *  autoSizeMax = 10,
     *  foreign_table_where = " AND sys_category.sys_language_uid = 0",
     *  wizards = {
     *      "_PADDING": 1,
     *      "_VERTICAL": 1,
     *      "add": {
     *          "type": "script",
     *          "title": "LLL::page_categories_add",
     *          "icon": "add.gif",
     *          "params": {
     *              "table": "sys_category",
     *              "pid": "###SITEROOT###",
     *              "setValue": "prepend"
     *          },
     *          "script": "wizard_add.php"
     *      },
     *      "list": {
     *          "type": "script",
     *          "title": "LLL::page_categories_list",
     *          "icon": "list.gif",
     *          "params": {
     *              "table": "sys_category",
     *              "pid": "###SITEROOT###"
     *          },
     *          "script": "wizard_list.php"
     *      }
     *  }
     * )
     *
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\IMIA\ImiaPageteaser\Domain\Model\Category>
     */
    protected $categories;

    /**
     * @SQL\Column(name="lastUpdated", create=false)
     * @TCA\DateTime(label="LLL::page_lastupdated")
     *
     * @var DateTime
     */
    protected $lastUpdated;

    /**
     * @SQL\Column(name="archivedate", type="int", length=11)
     * @TCA\DateTime(label="LLL::page_archivedate")
     *
     * @var DateTime
    */
    protected $archivedate;

    /**
     * @SQL\Column(type="int", length=11)
     * @TCA\Inline(
     *  foreign_table = "pages",
     *  foreign_field = "pid",
     *  maxitems = 9999
     * )
     *
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\IMIA\ImiaPageteaser\Domain\Model\Page>
     */
    protected $children;

    /**
     * @SQL\Column(type="int", length=11)
     * @TCA\Select(
     *  displayCond = "REC:NEW:true",
     *  itemsProcFunc = "IMIA\ImiaPageteaser\User\Page->getCreateLanguages",
     *  size = 1,
     *  default = "0",
     *  minitems = 1,
     *  maxitems = 1,
     * )
     *
     * @var integer
     */
    protected $createLanguage;

    /**
     * Constructor
     */
    public function __construct()
    {
        $this->categories = new ObjectStorage();
        $this->children = new ObjectStorage();
    }

    /**
     * @return \TYPO3\CMS\Core\Resource\FileReference
     * @FIXME How to do this a better way?
     */
    public function getTeaserImage()
    {
        if (!$this->teaserImage || !is_object($this->teaserImage)) {
            $files = $this->fileRepository->findByRelation('pages', 'teaser_image', $this->getUid());
            $this->teaserImage = array_shift($files);
        }

        return $this->teaserImage;
    }

    /**
     * @return array
     * @FIXME How to do this a better way?
     */
    public function getMedia()
    {
        if (!$this->media || (!is_object($this->media) && !is_array($this->media))) {
            $this->media = $this->fileRepository->findByRelation('pages', 'media', $this->getUid());
        }

        return $this->media;
    }

    /**
     * @return \TYPO3\CMS\Core\Resource\FileReference
     * @FIXME How to do this a better way?
     */
    public function getGalleryImages()
    {
        if (!$this->galleryImages || !is_object($this->galleryImages)) {
            $this->galleryImages = $this->fileRepository->findByRelation('pages', 'gallery_images', $this->getUid());
        }

        return $this->galleryImages;
    }

    /**
     * @return integer
     * @FIXME Very dirty, but is there a better way?
     */
    public function getContentFileRelations()
    {
        if (!$this->contentFileRelations) {
            $this->contentFileRelations = (int)@array_shift(
                $GLOBALS['TYPO3_DB']->exec_SELECTgetSingleRow(
                    'count(*) as count',
                    'sys_file_reference',
                    'tablenames="tt_content"' .
                        ' AND pid=' . $this->getUid() .
                        ' AND deleted=0' .
                        ' AND hidden=0'
                )
            );
        }

        return $this->contentFileRelations;
    }

    /**
     * @return \IMIA\ImiaPageteaser\Domain\Model\BackendUser
     */
    public function getCruser()
    {
        return $this->cruser;
    }

    /**
     * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\IMIA\ImiaPageteaser\Domain\Model\Category>
     */
    public function getCategories()
    {
        return $this->categories;
    }

    /**
     * @return \IMIA\ImiaPageteaser\Domain\Model\Category
     */
    public function getCategory()
    {
        $this->getCategories()->rewind();

        return $this->getCategories()->current();
    }

    /*
     * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\IMIA\ImiaPageteaser\Domain\Model\Page>
     */
    public function getTeaserPages()
    {
        if (!$this->teaserPageObjects) {
            $teaserPageUids = array_map('intval', explode(',', $this->teaserPages));
            $teaserPages = $this->pageRepository->findByUids($teaserPageUids);

            $teaserPageObjects = array();
            foreach($teaserPages as $teaserPage) {
                $teaserPageObjects[$teaserPage->getUid()] = $teaserPage;
            }

            $this->teaserPageObjects = array();
            foreach ($teaserPageUids as $teaserPageUid) {
                if (array_key_exists($teaserPageUid, $teaserPageObjects)) {
                    $this->teaserPageObjects[] = $teaserPageObjects[$teaserPageUid];
                }
            }
        }

        return $this->teaserPageObjects;
    }

    /**
     * @return \TYPO3\CMS\Core\Resource\FileReference
     * @FIXME How to do this a better way?
     */
    public function getGalleryPdf()
    {
        if (!$this->galleryPdf || !is_object($this->galleryPdf)) {
            $files = $this->fileRepository->findByRelation('pages', 'gallery_pdf', $this->getUid());
            $this->galleryPdf = array_shift($files);
        }

        return $this->galleryPdf;
    }

    /**
     * @return \IMIA\ImiaPageteaser\Domain\Model\Page
     */
    public function getPage()
    {
        return $this;
    }

    /**
     * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage
     */
    public function getChildren()
    {
        return $this->children;
    }

    /**
     * @return array
     */
    public function toArray()
    {
        $page = array();
        foreach (get_class_methods($this) as $method) {
            if (substr($method, 0, 3) === 'get') {
                $property = lcfirst(substr($method, 3));
                $value = $this->$method();

                if (is_object($value)) {
                    switch (get_class($value)) {
                        case 'DateTime':
                            $page[$property] = $value->getTimestamp();
                            break;
                        default:
                            try {
                                //$page[$property] = (string)$value;
                            } catch(\Exception $e) {};
                            break;
                    }
                } else {
                    $page[$property] = $value;
                }
            }
        }

        return $page;
    }

    /**
     * @return array
     */
    public function getTranslations()
    {
        if (!$this->translations) {
            $this->translations = array();
            foreach ($this->pageAltRepository->findByPage($this->uid) as $translation) {
                $this->translations[$translation->getRecordUid()] = $translation;
            }
        }

        return $this->translations;
    }

    /**
     * @param integer $sysLanguageUid
     * @return bool
     */
    public function hasTranslation($sysLanguageUid)
    {
        $hasTranslation = false;
        foreach ($this->getTranslations() as $translation) {
            if ($translation->getSysLanguageUid() == $sysLanguageUid) {
                $hasTranslation = true;
                break;
            }
        }

        return $hasTranslation;
    }

    /**
     * @return boolean
     */
    public function getHidden()
    {
        return $this->hidden;
    }

    /**
     * @return mixed
     */
    public function getStarttime()
    {
        return $this->starttime;
    }

    /**
     * @return \DateTime
     */
    public function getEndtime()
    {
        return $this->endtime;
    }

    /**
     * @return boolean
     */
    public function getIsLanguageVisible()
    {
        if (in_array($this->l18nCfg, array(1,3))) {
            return false;
        } else {
            return true;
        }
    }

    /**
     * @return \TYPO3\CMS\Core\Resource\FileReference
     * @FIXME How to do this a better way?
     */
    public function getFile()
    {
        if (!$this->file || !is_object($this->file)) {
            $this->file = null;
            $files = $this->fileRepository->findByRelation('pages', 'file', $this->getUid());
            if ($files && count($files) > 0) {
                $this->file = array_shift($files);
            }
        }

        return $this->file;
    }

    /**
     * @return DateTime
     */
    public function getArchivedate()
    {
        return $this->archivedate;
    }

    /**
     * @param DateTime $archivedate
     */
    public function setArchivedate($archivedate)
    {
        $this->archivedate = $archivedate;
    }
}